@@ -1,5 +1,6 @@
 #! /usr/bin/env python
 # -*- coding: utf-8 -*-
+"""TODO"""
 
 
 # This file is part of Linshare cli.
@@ -24,47 +25,62 @@
 #  Frédéric MARTIN frederic.martin.fma@gmail.com
 #
 
-from __future__ import unicode_literals
+
+
+import json
 
 from linshareapi.cache import Time
+from argtoolbox import DefaultCompleter as Completer
 from linsharecli.common.filters import PartialOr
 from linsharecli.admin.core import DefaultCommand
 from linsharecli.common.core import add_list_parser_options
 from linsharecli.common.core import add_delete_parser_options
-from argtoolbox import DefaultCompleter as Completer
+from linsharecli.common.actions import CreateAction
+from linsharecli.common.tables import DeleteAction
+from linsharecli.common.tables import TableBuilder
+from linsharecli.common.cell import ComplexCell
+from linsharecli.common.cell import ComplexCellBuilder
 
 
-# -----------------------------------------------------------------------------
 class DomainsCommand(DefaultCommand):
     """For  api >= 1.9"""
+    # pylint: disable=too-many-instance-attributes
     IDENTIFIER = "label"
     DEFAULT_SORT = "label"
-    DEFAULT_SORT_NAME = "label"
     RESOURCE_IDENTIFIER = "identifier"
 
     DEFAULT_TOTAL = "Domain found : %(count)s"
     MSG_RS_NOT_FOUND = "No domain could be found."
-    MSG_RS_DELETED = "%(position)s/%(count)s: The domain '%(label)s' (%(uuid)s) was deleted. (%(time)s s)"
-    MSG_RS_CAN_NOT_BE_DELETED = "The domain '%(label)s'  '%(uuid)s' can not be deleted."
+    MSG_RS_DELETED = (
+        "%(position)s/%(count)s: "
+        "The domain '%(label)s' (%(identifier)s) was deleted. (%(time)s s)"
+    )
+    MSG_RS_CAN_NOT_BE_DELETED = "The domain '%(label)s'  '%(identifier)s' can not be deleted."
     MSG_RS_CAN_NOT_BE_DELETED_M = "%(count)s domain (s) can not be deleted."
-    MSG_RS_UPDATED = "The domain '%(label)s' (%(uuid)s) was successfully updated. (%(time)s s)"
-    MSG_RS_CREATED = "The domain '%(label)s' (%(uuid)s) was successfully created. (%(time)s s)"
+    MSG_RS_UPDATED = "The domain '%(label)s' (%(identifier)s) was successfully updated."
+    MSG_RS_CREATED = (
+        "The domain '%(label)s' (%(identifier)s) "
+        "was successfully created. (%(_time)s s)"
+    )
 
+    # pylint: disable=no-self-use
     def init_old_language_key(self):
         """For  api >= 1.6 and api <= 1.8"""
-        self.IDENTIFIER = "identifier"
-        self.DEFAULT_SORT = "identifier"
-        self.RESOURCE_IDENTIFIER = "identifier"
-        self.DEFAULT_SORT_NAME = "identifier"
+        DomainsCommand.IDENTIFIER = "identifier"
+        DomainsCommand.DEFAULT_SORT = "identifier"
+        DomainsCommand.RESOURCE_IDENTIFIER = "identifier"
 
-        self.DEFAULT_TOTAL = "Domain found : %(count)s"
-        self.MSG_RS_NOT_FOUND = "No domain could be found."
-        self.MSG_RS_DELETED = "The domain '%(identifier)s' was deleted. (%(time)s s)"
-        self.MSG_RS_DELETED = "%(position)s/%(count)s: The domain '%(identifier)s' was deleted. (%(time)s s)"
-        self.MSG_RS_CAN_NOT_BE_DELETED = "The domain '%(identifier)s' can not be deleted."
-        self.MSG_RS_CAN_NOT_BE_DELETED_M = "%(count)s domain (s) can not be deleted."
-        self.MSG_RS_UPDATED = "The domain '%(identifier)s' was successfully updated."
-        self.MSG_RS_CREATED = "The domain '%(identifier)s' was successfully created."
+        DomainsCommand.DEFAULT_TOTAL = "Domain found : %(count)s"
+        DomainsCommand.MSG_RS_NOT_FOUND = "No domain could be found."
+        DomainsCommand.MSG_RS_DELETED = "The domain '%(identifier)s' was deleted. (%(time)s s)"
+        DomainsCommand.MSG_RS_DELETED = (
+            "%(position)s/%(count)s: "
+            "The domain '%(identifier)s' was deleted. (%(time)s s)"
+        )
+        DomainsCommand.MSG_RS_CAN_NOT_BE_DELETED = "The domain '%(identifier)s' can not be deleted."
+        DomainsCommand.MSG_RS_CAN_NOT_BE_DELETED_M = "%(count)s domain (s) can not be deleted."
+        DomainsCommand.MSG_RS_UPDATED = "The domain '%(identifier)s' was successfully updated."
+        DomainsCommand.MSG_RS_CREATED = "The domain '%(identifier)s' was successfully created."
 
     def complete(self, args, prefix):
         super(DomainsCommand, self).__call__(args)
@@ -74,50 +90,118 @@ class DomainsCommand(DefaultCommand):
         return (v.get(self.RESOURCE_IDENTIFIER)
                 for v in json_obj if v.get(self.RESOURCE_IDENTIFIER).startswith(prefix))
 
+    def complete_welcome(self, args, prefix):
+        """TODO"""
+        super(DomainsCommand, self).__call__(args)
+        json_obj = self.ls.welcome_messages.list(args.identifier)
+        return (v.get('uuid')
+                for v in json_obj if v.get('uuid').startswith(prefix))
+
+
+class ProviderCell(ComplexCell):
+    """TODO"""
+
+    def __unicode__(self):
+        if self.raw:
+            return str(self.value)
+        if self.value is None:
+            return self.none
+        output = []
+        for param in self.value:
+            display = (
+                "{baseDn} (ldap:{ldapConnectionUuid:.8}, "
+                "pattern:{userLdapPatternUuid:.8})"
+            )
+            output.append(display.format(**param))
+        return ",".join(output)
+
+
+class DDeleteAction(DeleteAction):
+    """TODO"""
+
+    MSG_RS_DELETED = (
+        "%(position)s/%(count)s: "
+        "The domain '%(label)s' (%(identifier)s) was deleted. (%(time)s s)"
+    )
+    MSG_RS_CAN_NOT_BE_DELETED = "The domain '%(identifier)s' can not be deleted."
+
+    def __init__(self):
+        super(DDeleteAction, self).__init__(
+            mode=0,
+            identifier="label",
+            resource_identifier="identifier",
+        )
 
 
-# -----------------------------------------------------------------------------
 class DomainsListCommand(DomainsCommand):
     """ List all domains."""
 
-    ACTIONS = {
-        'delete' : '_delete_all',
-        'count_only' : '_count_only',
-    }
-
     @Time('linsharecli.domains', label='Global time : %(time)s')
     def __call__(self, args):
         super(DomainsListCommand, self).__call__(args)
         if self.api_version == 0:
             self.init_old_language_key()
-        cli = self.ls.domains
-        table = self.get_table(args, cli, self.IDENTIFIER, args.fields)
-        json_obj = cli.list()
-        # Filters
-        filters = [PartialOr(self.IDENTIFIER, args.identifiers, True)]
-        return self._list(args, cli, table, json_obj, filters=filters)
+        endpoint = self.ls.domains
+        tbu = TableBuilder(self.ls, endpoint, self.DEFAULT_SORT)
+        tbu.add_action('delete', DDeleteAction())
+        tbu.load_args(args)
+        tbu.add_custom_cell("currentWelcomeMessage", ComplexCellBuilder('{name} ({uuid:.8})'))
+        tbu.add_custom_cell("providers", ProviderCell)
+        tbu.add_filters(
+            PartialOr(self.IDENTIFIER, args.identifiers, True),
+        )
+        return tbu.build().load_v2(endpoint.list()).render()
 
     def complete_fields(self, args, prefix):
+        """TODO"""
+        # pylint: disable=unused-argument
         super(DomainsListCommand, self).__call__(args)
         cli = self.ls.domains
         return cli.get_rbu().get_keys(True)
 
 
-# -----------------------------------------------------------------------------
-# Ignore unused variable prefix
-# pylint: disable-msg=W0613
 class DomainsCreateCommand(DomainsCommand):
     """ List all domains."""
+    # pylint: disable=unused-argument
 
     def __call__(self, args):
         super(DomainsCreateCommand, self).__call__(args)
-        rbu = self.ls.domains.get_rbu()
-        rbu.load_from_args(args)
-        return self._run(
-            self.ls.domains.create,
-            "The following domain '%(identifier)s' was successfully created",
-            args.identifier,
-            rbu.to_resource())
+        if self.api_version == 0:
+            self.init_old_language_key()
+        if not args.description:
+            args.description = args.identifier
+
+        act = CreateAction(self, self.ls.domains)
+        if act.load(args).execute():
+            if not args.domain_policy_auto:
+                return True
+            else:
+                domain_uuid = act.result.get('identifier')
+                policy = {
+                    "accessPolicy": {
+                        "rules": [
+                            {
+                                "domain": {
+                                    "identifier": domain_uuid,
+                                },
+                                "type": "ALLOW"
+                            },
+                            {
+                                "type": "DENY_ALL"
+                            }
+                        ]
+                    },
+                    "label": args.identifier,
+                }
+                self.log.debug(json.dumps(policy, sort_keys=True, indent=2))
+                policy = self.ls.domain_policies.create(policy)
+                self.log.debug(json.dumps(policy, sort_keys=True, indent=2))
+                domain = self.ls.domains.get(domain_uuid)
+                domain['policy'] = policy
+                self.ls.domains.update(domain)
+            return True
+        else:
+            return False
 
     def complete(self, args, prefix):
         super(DomainsCreateCommand, self).__call__(args)
@@ -146,7 +230,6 @@ class DomainsCreateCommand(DomainsCommand):
         return self.ls.domains.options_mail_language()
 
 
-# -----------------------------------------------------------------------------
 class DomainsUpdateCommand(DomainsCommand):
     """ Update a domain."""
 
@@ -162,11 +245,17 @@ class DomainsUpdateCommand(DomainsCommand):
         rbu = self.ls.domains.get_rbu()
         rbu.copy(resource)
         rbu.load_from_args(args)
+        resource = rbu.to_resource()
+        if hasattr(args, 'current_welcome_message'):
+            if args.current_welcome_message:
+                resource['currentWelcomeMessage'] = {
+                    'uuid': args.current_welcome_message
+                }
         return self._run(
             self.ls.domains.update,
             "The following domain '%(identifier)s' was successfully updated",
             args.identifier,
-            rbu.to_resource())
+            resource)
 
     def complete(self, args, prefix):
         super(DomainsUpdateCommand, self).__call__(args)
@@ -176,16 +265,17 @@ class DomainsUpdateCommand(DomainsCommand):
 
     def complete_role(self, args, prefix):
         """ Complete with available role."""
+        # pylint: disable=unused-argument
         super(DomainsUpdateCommand, self).__call__(args)
         return self.ls.domains.options_role()
 
     def complete_language(self, args, prefix):
         """ Complete with available language."""
+        # pylint: disable=unused-argument
         super(DomainsUpdateCommand, self).__call__(args)
         return self.ls.domains.options_language()
 
 
-# -----------------------------------------------------------------------------
 class DomainsDeleteCommand(DomainsCommand):
     """ List all domains."""
 
@@ -198,11 +288,9 @@ class DomainsDeleteCommand(DomainsCommand):
                 args,
                 cli,
                 args.identifier)
-        else:
-            return self._delete_all(args, cli, args.uuids)
+        return self._delete_all(args, cli, args.uuids)
 
 
-# -----------------------------------------------------------------------------
 class DomainProvidersCreateCommand(DomainsCommand):
     """ Update a domain."""
 
@@ -220,9 +308,15 @@ class DomainProvidersCreateCommand(DomainsCommand):
         rbu.load_from_args(args)
         providers = [{
             "baseDn": args.basedn,
-            "domainPatternId": args.dpattern,
-            "ldapConnectionId": args.ldap
+            "userLdapPatternUuid": args.dpattern,
+            "ldapConnectionUuid": args.ldap
         }]
+        if self.api_version < 2:
+            providers = [{
+                "baseDn": args.basedn,
+                "domainPatternId": args.dpattern,
+                "ldapConnectionId": args.ldap
+            }]
         rbu.set_value("providers", providers)
         return self._run(
             self.ls.domains.update,
@@ -238,19 +332,20 @@ successfully created",
                 for v in json_obj if v.get('identifier').startswith(prefix))
 
     def complete_ldap(self, args, prefix):
+        """TODO"""
         super(DomainProvidersCreateCommand, self).__call__(args)
         json_obj = self.ls.ldap_connections.list()
         return (v.get('identifier')
                 for v in json_obj if v.get('identifier').startswith(prefix))
 
     def complete_dpattern(self, args, prefix):
+        """TODO"""
         super(DomainProvidersCreateCommand, self).__call__(args)
         json_obj = self.ls.domain_patterns.list()
         return (v.get('identifier')
                 for v in json_obj if v.get('identifier').startswith(prefix))
 
 
-# -----------------------------------------------------------------------------
 class DomainProvidersDeleteCommand(DomainsCommand):
     """ List all domains."""
 
@@ -282,7 +377,6 @@ deleted",
                 for v in json_obj if v.get('identifier').startswith(prefix))
 
 
-# -----------------------------------------------------------------------------
 class DomainProvidersUpdateCommand(DomainsCommand):
     """ Update a user provider."""
 
@@ -299,11 +393,16 @@ class DomainProvidersUpdateCommand(DomainsCommand):
         rbu.copy(resource)
         rbu.load_from_args(args)
         resource = rbu.to_resource()
+        ldap_uuid = "ldapConnectionUuid"
+        ldap_pattern_uuid = "userLdapPatternUuid"
+        if self.api_version < 2:
+            ldap_uuid = "ldapConnectionId"
+            ldap_pattern_uuid = "domainPatternId"
         for i in resource['providers']:
             if args.ldap is not None:
-                i['ldapConnectionId'] = args.ldap
+                i[ldap_uuid] = args.ldap
             if args.dpattern is not None:
-                i['domainPatternId'] = args.dpattern
+                i[ldap_pattern_uuid] = args.dpattern
             if args.basedn is not None:
                 i['baseDn'] = args.basedn
         return self._run(
@@ -320,20 +419,23 @@ updated",
                 for v in json_obj if v.get('identifier').startswith(prefix))
 
     def complete_ldap(self, args, prefix):
+        """TODO"""
         super(DomainProvidersUpdateCommand, self).__call__(args)
         json_obj = self.ls.ldap_connections.list()
         return (v.get('identifier')
                 for v in json_obj if v.get('identifier').startswith(prefix))
 
     def complete_dpattern(self, args, prefix):
+        """TODO"""
         super(DomainProvidersUpdateCommand, self).__call__(args)
         json_obj = self.ls.domain_patterns.list()
         return (v.get('identifier')
                 for v in json_obj if v.get('identifier').startswith(prefix))
 
-# -----------------------------------------------------------------------------
+
 def add_parser(subparsers, name, desc, config):
     """Add all domain sub commands."""
+    # pylint: disable=too-many-statements
     parser_tmp = subparsers.add_parser(name, help=desc)
     subparsers2 = parser_tmp.add_subparsers()
     api_version = config.server.api_version.value
@@ -346,7 +448,7 @@ def add_parser(subparsers, name, desc, config):
         'identifiers', nargs="*",
         help="Filter domains by their identifiers")
     parser.add_argument('-n', '--label', action="store_true",
-                             help="sort by domain label")
+                        help="sort by domain label")
     if api_version == 0:
         add_list_parser_options(parser, delete=True, cdate=False)
     else:
@@ -377,9 +479,13 @@ def add_parser(subparsers, name, desc, config):
     parser_tmp2.add_argument(
         '--domain-policy', dest="domain_policy", action="store",
         help="TODO").completer = Completer("complete_policy")
+    parser_tmp2.add_argument(
+        '--domain-policy-auto', dest="domain_policy_auto", action="store_true",
+        help="TODO")
     parser_tmp2.add_argument(
         '--mail-config', dest="mail_config", action="store",
         help="TODO").completer = Completer("complete_mail")
+    parser_tmp2.add_argument('--cli-mode', action="store_true", help="")
     parser_tmp2.set_defaults(__func__=DomainsCreateCommand(config))
 
     # command : update
@@ -399,6 +505,10 @@ def add_parser(subparsers, name, desc, config):
         parser_tmp2.add_argument(
             '--mail-language', dest="external_mail_locale", action="store",
             help="").completer = Completer("complete_mail_language")
+    parser_tmp2.add_argument(
+        '--welcome', dest="current_welcome_message", action="store",
+        help="welcome message identifier").completer = Completer(
+            "complete_welcome")
     parser_tmp2.set_defaults(__func__=DomainsUpdateCommand(config))
 
     # command : delete
